@@ -253,6 +253,31 @@ func ignoreOptional(t *schema.OptionalType, requireInitializers bool) bool {
 	return false
 }
 
+func simplifyInputUnion(union *schema.UnionType) *schema.UnionType {
+	elements := make([]schema.Type, len(union.ElementTypes))
+	for i, et := range union.ElementTypes {
+		if input, ok := et.(*schema.InputType); ok {
+			switch input.ElementType.(type) {
+			case *schema.ArrayType, *schema.MapType:
+				// Instead of just replacing Input<{Array,Map}<T>> with {Array,Map}<T>, replace it with
+				// {Array,Map}<Plain(T)>. This matches the behavior of typeString when presented with an
+				// Input<{Array,Map}<T>>.
+				elements[i] = codegen.PlainType(input.ElementType)
+			default:
+				elements[i] = input.ElementType
+			}
+		} else {
+			elements[i] = et
+		}
+	}
+	return &schema.UnionType{
+		ElementTypes:  elements,
+		DefaultType:   union.DefaultType,
+		Discriminator: union.Discriminator,
+		Mapping:       union.Mapping,
+	}
+}
+
 func (mod *modContext) unionTypeString(t *schema.UnionType, qualifier string, input, wrapInput, state, requireInitializers bool) string {
 	elementTypeSet := stringSet{}
 	var elementTypes []string
@@ -310,7 +335,7 @@ func (mod *modContext) typeString(t schema.Type, qualifier string, input, state,
 		}
 
 		if union, ok := elem.(*schema.UnionType); ok {
-			union = codegen.SimplifyInputUnion(union).(*schema.UnionType)
+			union = simplifyInputUnion(union)
 			if inputType == "Input" {
 				return mod.unionTypeString(union, qualifier, input, true, state, requireInitializers)
 			}
@@ -426,8 +451,16 @@ var docCommentEscaper = strings.NewReplacer(
 	`>`, "&gt;",
 )
 
-func printComment(w io.Writer, comment string, indent string) {
-	lines := strings.Split(docCommentEscaper.Replace(comment), "\n")
+func printComment(w io.Writer, comment, indent string) {
+	printCommentWithOptions(w, comment, indent, true /*escape*/)
+}
+
+func printCommentWithOptions(w io.Writer, comment, indent string, escape bool) {
+	if escape {
+		comment = docCommentEscaper.Replace(comment)
+	}
+
+	lines := strings.Split(comment, "\n")
 	for len(lines) > 0 && lines[len(lines)-1] == "" {
 		lines = lines[:len(lines)-1]
 	}
@@ -446,6 +479,7 @@ type plainType struct {
 	res                   *schema.Resource
 	name                  string
 	comment               string
+	unescapeComment       bool
 	baseClass             string
 	propertyTypeQualifier string
 	properties            []*schema.Property
@@ -578,7 +612,7 @@ func (pt *plainType) genInputType(w io.Writer, level int) error {
 	}
 
 	// Open the class.
-	printComment(w, pt.comment, indent)
+	printCommentWithOptions(w, pt.comment, indent, !pt.unescapeComment)
 	fmt.Fprintf(w, "%spublic %sclass %s : Pulumi.%s\n", indent, sealed, pt.name, pt.baseClass)
 	fmt.Fprintf(w, "%s{\n", indent)
 
@@ -615,6 +649,7 @@ func (pt *plainType) genOutputType(w io.Writer, level int) {
 	fmt.Fprintf(w, "\n")
 
 	// Open the class and attribute it appropriately.
+	printCommentWithOptions(w, pt.comment, indent, !pt.unescapeComment)
 	fmt.Fprintf(w, "%s[OutputType]\n", indent)
 	fmt.Fprintf(w, "%spublic sealed class %s\n", indent, pt.name)
 	fmt.Fprintf(w, "%s{\n", indent)
@@ -1012,6 +1047,58 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 		fmt.Fprintf(w, "        }\n")
 	}
 
+	// Generate methods.
+	genMethod := func(method *schema.Method) {
+		methodName := Title(method.Name)
+		fun := method.Function
+
+		fmt.Fprintf(w, "\n")
+
+		returnType, typeParameter := "void", ""
+		if fun.Outputs != nil {
+			typeParameter = fmt.Sprintf("<%s%sResult>", className, methodName)
+			returnType = fmt.Sprintf("Pulumi.Output%s", typeParameter)
+		}
+
+		var argsParamDef string
+		argsParamRef := "CallArgs.Empty"
+		if fun.Inputs != nil {
+			var hasArgs bool
+			allOptionalInputs := true
+			for _, arg := range fun.Inputs.InputShape.Properties {
+				if arg.Name == "__self__" {
+					continue
+				}
+				hasArgs = true
+				allOptionalInputs = allOptionalInputs && !arg.IsRequired()
+			}
+			if hasArgs {
+				var argsDefault, sigil string
+				if allOptionalInputs {
+					// If the number of required input properties was zero, we can make the args object optional.
+					argsDefault, sigil = " = null", "?"
+				}
+
+				argsParamDef = fmt.Sprintf("%s%sArgs%s args%s", className, methodName, sigil, argsDefault)
+				argsParamRef = fmt.Sprintf("args ?? new %s%sArgs()", className, methodName)
+			}
+		}
+
+		// Emit the doc comment, if any.
+		printComment(w, fun.Comment, "        ")
+
+		if fun.DeprecationMessage != "" {
+			fmt.Fprintf(w, "        [Obsolete(@\"%s\")]\n", strings.ReplaceAll(fun.DeprecationMessage, `"`, `""`))
+		}
+
+		fmt.Fprintf(w, "        public %s %s(%s)\n", returnType, methodName, argsParamDef)
+		fmt.Fprintf(w, "            => Pulumi.Deployment.Instance.Call%s(\"%s\", %s, this);\n",
+			typeParameter, fun.Token, argsParamRef)
+	}
+	for _, method := range r.Methods {
+		genMethod(method)
+	}
+
 	// Close the class.
 	fmt.Fprintf(w, "    }\n")
 
@@ -1056,6 +1143,70 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 		}
 	}
 
+	// Generate method types.
+	genMethodTypes := func(method *schema.Method) error {
+		methodName := Title(method.Name)
+		fun := method.Function
+
+		// Generate args type.
+		var args []*schema.Property
+		if fun.Inputs != nil {
+			// Filter out the __self__ argument from the inputs.
+			args = make([]*schema.Property, 0, len(fun.Inputs.InputShape.Properties)-1)
+			for _, arg := range fun.Inputs.InputShape.Properties {
+				if arg.Name == "__self__" {
+					continue
+				}
+				args = append(args, arg)
+			}
+		}
+		if len(args) > 0 {
+			comment, escape := fun.Inputs.Comment, true
+			if comment == "" {
+				comment, escape = fmt.Sprintf(
+					"The set of arguments for the <see cref=\"%s.%s\"/> method.", className, methodName), false
+			}
+			argsType := &plainType{
+				mod:                   mod,
+				comment:               comment,
+				unescapeComment:       !escape,
+				name:                  fmt.Sprintf("%s%sArgs", className, methodName),
+				baseClass:             "CallArgs",
+				propertyTypeQualifier: "Inputs",
+				properties:            args,
+				args:                  true,
+			}
+			if err := argsType.genInputType(w, 1); err != nil {
+				return err
+			}
+		}
+
+		// Generate result type.
+		if fun.Outputs != nil {
+			comment, escape := fun.Inputs.Comment, true
+			if comment == "" {
+				comment, escape = fmt.Sprintf(
+					"The results of the <see cref=\"%s.%s\"/> method.", className, methodName), false
+			}
+			resultType := &plainType{
+				mod:                   mod,
+				comment:               comment,
+				unescapeComment:       !escape,
+				name:                  fmt.Sprintf("%s%sResult", className, methodName),
+				propertyTypeQualifier: "Outputs",
+				properties:            fun.Outputs.Properties,
+			}
+			resultType.genOutputType(w, 1)
+		}
+
+		return nil
+	}
+	for _, method := range r.Methods {
+		if err := genMethodTypes(method); err != nil {
+			return err
+		}
+	}
+
 	// Close the namespace.
 	fmt.Fprintf(w, "}\n")
 
@@ -1332,6 +1483,10 @@ var pulumiImports = []string{
 }
 
 func (mod *modContext) getTypeImports(t schema.Type, recurse bool, imports map[string]codegen.StringSet, seen codegen.Set) {
+	mod.getTypeImportsForResource(t, recurse, imports, seen, nil)
+}
+
+func (mod *modContext) getTypeImportsForResource(t schema.Type, recurse bool, imports map[string]codegen.StringSet, seen codegen.Set, res *schema.Resource) {
 	if seen.Has(t) {
 		return
 	}
@@ -1362,6 +1517,11 @@ func (mod *modContext) getTypeImports(t schema.Type, recurse bool, imports map[s
 			return
 		}
 
+		// Don't import itself.
+		if t.Resource == res {
+			return
+		}
+
 		modName, name, modPath := mod.pkg.TokenToModule(t.Token), tokenToName(t.Token), ""
 		if modName != mod.mod {
 			mp, err := filepath.Rel(mod.mod, modName)
@@ -1392,6 +1552,10 @@ func (mod *modContext) getTypeImports(t schema.Type, recurse bool, imports map[s
 }
 
 func (mod *modContext) getImports(member interface{}, imports map[string]codegen.StringSet) {
+	mod.getImportsForResource(member, imports, nil)
+}
+
+func (mod *modContext) getImportsForResource(member interface{}, imports map[string]codegen.StringSet, res *schema.Resource) {
 	seen := codegen.Set{}
 	switch member := member.(type) {
 	case *schema.ObjectType:
@@ -1404,10 +1568,22 @@ func (mod *modContext) getImports(member interface{}, imports map[string]codegen
 		return
 	case *schema.Resource:
 		for _, p := range member.Properties {
-			mod.getTypeImports(p.Type, false, imports, seen)
+			mod.getTypeImportsForResource(p.Type, false, imports, seen, res)
 		}
 		for _, p := range member.InputProperties {
-			mod.getTypeImports(p.Type, false, imports, seen)
+			mod.getTypeImportsForResource(p.Type, false, imports, seen, res)
+		}
+		for _, method := range member.Methods {
+			if method.Function.Inputs != nil {
+				for _, p := range method.Function.Inputs.Properties {
+					mod.getTypeImportsForResource(p.Type, false, imports, seen, res)
+				}
+			}
+			if method.Function.Outputs != nil {
+				for _, p := range method.Function.Outputs.Properties {
+					mod.getTypeImportsForResource(p.Type, false, imports, seen, res)
+				}
+			}
 		}
 		return
 	case *schema.Function:
@@ -1667,7 +1843,7 @@ func (mod *modContext) gen(fs fs) error {
 	// Resources
 	for _, r := range mod.resources {
 		imports := map[string]codegen.StringSet{}
-		mod.getImports(r, imports)
+		mod.getImportsForResource(r, imports, r)
 
 		buffer := &bytes.Buffer{}
 		var additionalImports []string
@@ -1962,7 +2138,9 @@ func generateModuleContextMap(tool string, pkg *schema.Package) (map[string]*mod
 	// Find input and output types referenced by functions.
 	for _, f := range pkg.Functions {
 		mod := getModFromToken(f.Token, pkg)
-		mod.functions = append(mod.functions, f)
+		if !f.IsMethod {
+			mod.functions = append(mod.functions, f)
+		}
 		if f.Inputs != nil {
 			visitObjectTypes(f.Inputs.Properties, func(t *schema.ObjectType) {
 				details := getModFromToken(t.Token, t.Package).details(t)
