@@ -3,80 +3,22 @@ package gen
 import (
 	"bytes"
 	"io/ioutil"
-	"os"
 	"path/filepath"
 	"testing"
 
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
-
 	"github.com/pulumi/pulumi/pkg/v3/codegen"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/hcl2"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/hcl2/model/format"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/hcl2/syntax"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/test"
+	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/utils"
+	"github.com/stretchr/testify/assert"
 )
 
 var testdataPath = filepath.Join("..", "internal", "test", "testdata")
 
-func TestGenProgram(t *testing.T) {
-	files, err := ioutil.ReadDir(testdataPath)
-	if err != nil {
-		t.Fatalf("could not read test data: %v", err)
-	}
-
-	for _, f := range files {
-		if filepath.Ext(f.Name()) != ".pp" {
-			continue
-		}
-
-		if filepath.Base(f.Name()) == "azure-native.pp" {
-			// The generated code fails to compile
-			continue
-		}
-
-		t.Run(f.Name(), func(t *testing.T) {
-			path := filepath.Join(testdataPath, f.Name())
-			contents, err := ioutil.ReadFile(path)
-			if err != nil {
-				t.Fatalf("could not read %v: %v", path, err)
-			}
-			expected, err := ioutil.ReadFile(path + ".go")
-			if err != nil {
-				t.Fatalf("could not read %v: %v", path+".go", err)
-			}
-
-			parser := syntax.NewParser()
-			err = parser.ParseFile(bytes.NewReader(contents), f.Name())
-			if err != nil {
-				t.Fatalf("could not read %v: %v", path, err)
-			}
-			if parser.Diagnostics.HasErrors() {
-				t.Fatalf("failed to parse files: %v", parser.Diagnostics)
-			}
-
-			program, diags, err := hcl2.BindProgram(parser.Files, hcl2.PluginHost(test.NewHost(testdataPath)))
-			if err != nil {
-				t.Fatalf("could not bind program: %v", err)
-			}
-			if diags.HasErrors() {
-				t.Fatalf("failed to bind program: %v", diags)
-			}
-
-			files, diags, err := GenerateProgram(program)
-			assert.NoError(t, err)
-			if diags.HasErrors() {
-				t.Fatalf("failed to generate program: %v", diags)
-			}
-
-			if os.Getenv("PULUMI_ACCEPT") != "" {
-				err := ioutil.WriteFile(path+".go", files["main.go"], 0600)
-				require.NoError(t, err)
-			}
-
-			assert.Equal(t, string(expected), string(files["main.go"]))
-		})
-	}
+func TestGenerateProgram(t *testing.T) {
+	test.TestProgramCodegen(t, "go", GenerateProgram)
 }
 
 func TestCollectImports(t *testing.T) {
@@ -117,7 +59,7 @@ func newTestGenerator(t *testing.T, testFile string) *generator {
 			t.Fatalf("failed to parse files: %v", parser.Diagnostics)
 		}
 
-		program, diags, err := hcl2.BindProgram(parser.Files, hcl2.PluginHost(test.NewHost(testdataPath)))
+		program, diags, err := hcl2.BindProgram(parser.Files, hcl2.PluginHost(utils.NewHost(testdataPath)))
 		if err != nil {
 			t.Fatalf("could not bind program: %v", err)
 		}
