@@ -618,6 +618,10 @@ func TestConstructMethodsGo(t *testing.T) {
 	}
 }
 
+func TestConstructMethodsUnknownGo(t *testing.T) {
+	testConstructMethodsUnknown(t, "go", "github.com/pulumi/pulumi/sdk/v3")
+}
+
 func TestConstructProviderGo(t *testing.T) {
 	const testDir = "construct_component_provider"
 	tests := []struct {
