@@ -1040,6 +1040,10 @@ func TestConstructMethodsNode(t *testing.T) {
 	}
 }
 
+func TestConstructMethodsUnknownNode(t *testing.T) {
+	testConstructMethodsUnknown(t, "nodejs", "@pulumi/pulumi")
+}
+
 func TestConstructProviderNode(t *testing.T) {
 	const testDir = "construct_component_provider"
 	tests := []struct {
